% setfocus -- set input focus.
%
%      setfocus(h)
%
% Set the input focus on Handle Graphics object H.

%% setfocus.m --- the preceding comment is the documentation string.

% Copyright (C) 2005 Ralph Schleicher

% This file is part of MTk, the Matlab widget toolkit.

% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions
% are met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the
%      distribution.
%
%    * Neither the name of the copyright holder nor the names of its
%      contributors may be used to endorse or promote products derived
%      from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
% FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
% COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
% INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
% CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
% LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
% ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

%% Commentary:

% This is an improved version of Martin Weber's 'setfocus' command
% from Matlab Central.

%% Code:

% Program entry point.
function setfocus(h)

  if nargin < 1
    error(nargchk(1, 1, nargin));
  end

  if numel(h) ~= 1
    error('Invalid argument');
  end

  % Get figure handle.
  fig = get(h, 'Parent');
  if isempty(fig)
    return;
  elseif fig == 0
    fig = h;
  end

  % Raise figure.
  figure(fig);

  % Matlab version 7 has built-in support.
  if sscanf(version, '%d') > 6
    if h ~= fig
      uicontrol(h);
    end
    return;
  end

  % Determine the focus point.
  units = get(fig, 'Units');
  set(fig, 'Units', 'pixels');
  fig_pos = get(fig, 'Position');
  set(fig, 'Units', units);
  % Lower left corner of the figure window.
  point = fig_pos(1:2);
  if fig ~= h
    units = get(h, 'Units');
    set(h, 'Units', 'pixels');
    obj_pos = get(h, 'Position');
    set(h, 'Units', units);
    % Upper left corner of the Handle Graphics object.
    point = point + obj_pos(1:2) + [1 obj_pos(4)] - 2;
    if ispc
      % Get off by 2.
      point = point + [2 -2];
    end
  end

  % Save call-back handler.
  try
    callback = get(h, 'Callback');
    if ~ isempty(callback)
      set(h, 'Callback', '');
    end
  catch
    callback = [];
  end

  % Save pointer location.
  mark = get(0, 'PointerLocation');

  % Click on the focus point.
  units = get(0, 'Units');
  set(0, 'Units', 'pixels');
  set(0, 'PointerLocation', point);
  set(0, 'Units', units);

  try
    drawnow;
    mouseclick;
  end

  % Restore pointer location.
  set(0, 'PointerLocation', mark);

  % Restore call-back handler.
  if ~ isempty(callback)
    set(h, 'Callback', callback);
  end

%% setfocus.m ends here
