% tango -- Tango desktop project utility.
%
%      [...] = tango(command, ...)
%
% The 'tango' command provides support for some elements of the
% Tango desktop project.
%
% First argument COMMAND is a string.  Below is a list of all
% public commands together with their meanings.
%
%   help              Display help text.
%   handle            Return function handle.
%   icon              Return icon file name.
%   groups            Return color group information.
%   gradient          Build smooth color maps.
%   color             Map color name to color value.
%   yellow            Return shades of yellow.
%   orange            Return shades of orange.
%   red               Return shades of red.
%   purple            Return shades of purple.
%   blue              Return shades of blue.
%   green             Return shades of green.
%   brown             Return shades of brown.
%   light_gray        Return shades of light gray.
%   dark_gray         Return shades of dark gray.
%   black             Return black color value.
%   white             Return white color value.
%
% Enter 'tango help COMMAND' to display the detailed help text
% for a command.
%
% See also: mtk

%% tango.m --- the preceding comment is the documentation string.

% Copyright (C) 2008 Ralph Schleicher

% This file is part of MTk, the Matlab widget toolkit.

% Redistribution and use in source and binary forms, with or without
% modification, are permitted provided that the following conditions
% are met:
%
%    * Redistributions of source code must retain the above copyright
%      notice, this list of conditions and the following disclaimer.
%
%    * Redistributions in binary form must reproduce the above copyright
%      notice, this list of conditions and the following disclaimer in
%      the documentation and/or other materials provided with the
%      distribution.
%
%    * Neither the name of the copyright holder nor the names of its
%      contributors may be used to endorse or promote products derived
%      from this software without specific prior written permission.
%
% THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
% "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
% LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS
% FOR A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE
% COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT,
% INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING,
% BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
% LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
% CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
% LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
% ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
% POSSIBILITY OF SUCH DAMAGE.

%% Code:

% Program entry point.
function varargout = tango(varargin)

  try
    % Call subfunction quickly.
    f = str2func(['tango_', varargin{1}]);
    if nargout == 0
      f(varargin{2:end});
    else
      varargout = cell(1, nargout);
      [varargout{:}] = f(varargin{2:end});
    end
  catch
    if exist('rethrow', 'builtin')
      rethrow(mtk('suberr', lasterror, 'tango_', varargin));
    else
      error(mtk('suberr', lasterr, 'tango_', varargin));
    end
  end

% tango/help -- display help text.
%
%      tango('help', topic)
%      tango('help')
%
% Display the help text for a Tango command.
%
% If the first argument TOPIC is omitted, display the help text for
% the gateway command.
function tango_help(topic)

  if nargin < 1
    topic = [];
  end

  mtk('help', 'tango', topic, mfilename);

% tango/handle -- get function handle.
%
%      h = tango('handle', command, ...)
%
% Return a function handle to the specified Tango command.
%
% Optional first argument COMMAND is the name of a Tango command.
% Remaining arguments are subcommand names.
%
% Return value H is the function handle of the Tango command.
function h = tango_handle(varargin)

  % Build function name.
  tem = {'_'};

  str = [tem(ones(1, nargin)); varargin];
  str = ['tango', str{:}];

  h = str2func(str);

% tango/icon -- lookup icon by name.
%
%      filename = tango('icon', theme, name, size)
%      filename = tango('icon', name, size)
%      filename = tango('icon', name)
%
% Lookup icon file name by icon name.
%
% Optional first argument THEME is the name of the icon theme to
%  be used.  If THEME is omitted or empty, utilize the configured
%  default icon theme.
% Second argument NAME is the name of the icon to search for.
% Optional third argument SIZE is the desired icon size in pixel.
%  Value has to be a non-negative integral number.  Typical icon
%  sizes are 16, 22, or 24.  If SIZE is omitted, empty, or zero,
%  no specific icon size is searched for.
%
% Return value is the icon file name.  It is not guaranteed that a
% file with that name exists.
%
% For example,
%
%      [im, map] = imload(tango('icon', 'go-home', 16));
%
% See also: mico
function filename = tango_icon(varargin)

  % Check arguments.
  switch nargin
   case 1
    theme = [];
    name = varargin{1};
    siz = 0;
   case 2
    theme = [];
    name = varargin{1};
    siz = varargin{2};
   case 3
    theme = varargin{1};
    name = varargin{2};
    siz = varargin{3};
   otherwise
    error(nargchk(1, 3, nargin));
  end

  default_theme = 'Tango';
  if isempty(theme)
    theme = default_theme;
  elseif ~ mtk('stringp', theme)
    error('Icon theme name has to be a non-empty string');
  end

  if isempty(name) || ~ mtk('stringp', name)
    error('Icon name has to be a non-empty string');
  end
  if ispc
    name = strrep(name, '/', filesep);
  end

  if isempty(siz)
    siz = 0;
  elseif ~ mtk('integerp', siz) || siz < 0
    error('Icon size has to be a non-negative integral number');
  end
  if siz > 0
    sizdir = sprintf('%dx%d', siz, siz);
  else
    sizdir = '';
  end

  % Icon themes to be searched.
  themes = {theme};
  if ~ strcmp(theme, default_theme)
    themes{end + 1} = default_theme;
  end
  if ~ strcmp(theme, 'hicolor')
    themes{end + 1} = 'hicolor';
  end
  themes{end + 1} = '';

  % Icon directories to be searched.
  icon_dir = tango_icon_dir;

  % Icon file formats to be searched.
  formats = {'.png'};

  for i = 1:numel(themes)
    theme = themes{i};

    for j = 1:numel(icon_dir)
      prefix = icon_dir{j};

      for k = 1:numel(formats)
        ext = formats{k};

        filename = fullfile(prefix, theme, sizdir, [name, ext]);
        if exist(filename, 'file')
          return;
        end
      end
    end
  end

  % Not found.
  filename = [];

% Return a list of directories that may contain icons.
function icon_dir = tango_icon_dir

  % Return value.
  icon_dir = {};

  % Add '$HOME/.icons' for backward compatibility.
  str = getenv('HOME');
  if ~ isempty(str)
    if ispc
      str = strrep(str, '/', filesep);
    end
    str = fullfile(str, '.icons');
    if mtk('absolutep', str) && exist(str, 'dir')
      icon_dir{end + 1} = str;
    end
  end

  % Add the 'icons' directories from the standard search path.
  str = getenv('XDG_DATA_DIRS');
  if ~ isempty(str)
    if ispc
      str = strrep(str, '/', filesep);
    end
    % Split search path.
    str = strrep(str, '''', '''''');
    str = strrep(str, pathsep, '''; ''');
    str = evalin('base', ['{''', str, '''}']);
    for i = 1:numel(str)
      str{i} = fullfile(str{i}, 'icons');
      if mtk('implicitp', str{i})
        str{i} = fullfile('.', str{i});
      end
      if exist(str{i}, 'dir')
        icon_dir{end + 1} = str{i};
      end
    end
  end

  if isunix
    str = '/usr/share/pixmaps';
    if exist(str, 'dir')
      icon_dir{end + 1} = str;
    end
  end

  % Add the 'icons' directories from the Matlab search path.
  str = path;
  str = strrep(str, '''', '''''');
  str = strrep(str, pathsep, '''; ''');
  str = evalin('base', ['{''', str, '''}']);
  % Remove Matlab installation directories.
  str(strncmp(str, [matlabroot, filesep], numel(matlabroot) + 1)) = [];
  for i = 1:numel(str)
    str{i} = fullfile(str{i}, 'icons');
    if mtk('implicitp', str{i})
      str{i} = fullfile('.', str{i});
    end
    if exist(str{i}, 'dir')
      icon_dir{end + 1} = str{i};
    end
  end

  str = fullfile(matlabroot, 'toolbox', 'matlab', 'icons');
  if exist(str, 'dir')
    icon_dir{end + 1} = str;
  else
    str = fullfile(matlabroot, 'toolbox', 'matlab', 'general');
    if exist(str, 'dir')
      icon_dir{end + 1} = str;
    end
  end

  % Add the 'icons' directory relative to this file.
  str = fullfile(fileparts(which(mfilename)), 'icons');
  if exist(str, 'dir')
    icon_dir{end + 1} = str;
  end

  icon_dir = icon_dir(:);

% tango/groups -- color groups.
%
%      [str, num] = tango('groups')
%
% Return color group information.
%
% First return value STR are the Tango color group names.  Value
%  is a cell array of strings.  Each array element is the name of
%  a Tango color group in lowercase letters, for example 'butter'.
% Second return value NUM are the number of colors defined in each
%  group.  Value is a vector of positive integral numbers.
function [str, num] = tango_groups

  persistent p_str p_num
  if isempty(p_str)
    dat = {'butter',      3; ...
           'orange',      3; ...
           'scarlet red', 3; ...
           'plum',        3; ...
           'sky blue',    3; ...
           'chameleon',   3; ...
           'chocolate',   3; ...
           'aluminium'    6};

    p_str = dat(:, 1);
    p_num = [dat{:, 2}]';
  end

  str = p_str;
  num = p_num;

% tango/gradient -- color gradients.
%
%      map = tango('gradient', m, name, ...)
%      map = tango('gradient', name, ...)
%
% Build a color map that varies smoothly between the given colors.
%
% Optional first argument M is the number of colors for each
%  individual gradient.  Default is to get a total number of
%  approx. 64 colors.
% Remaining arguments are one or more color names.
%
% Return value is a color map.
%
% For example,
%
%      colormap(tango('gradient', 'yellow', 'orange', 'red'))
%
% sets the color map of the current figure to vary smoothly from
% yellow through orange to red.
%
% See also: colormap
function map = tango_gradient(varargin)

  if nargin < 1
    error(nargchk(1, inf, nargin));
  end

  if ischar(varargin{1})
    % Length of each individual gradient.
    if nargin == 1
      m = 64;
    else
      m = max(8, fix(64 / (nargin - 1))) + (nargin == 6);
    end
  else
    if nargin < 2
      error(nargchk(2, inf, nargin));
    end
    m = varargin{1};
    varargin(1) = [];
    if ~ mtk('integerp', m) || m < 2
      error('Invalid argument');
    end
  end

  % Number of gradients.
  n = numel(varargin) - 1;
  switch n
   case 0
    map = ones(m, 1) * tango_color(varargin{1});
   case 1
    map = tango1_gradient(varargin{1}, varargin{2}, m);
   otherwise
    map = cell(1, n);
    map{1} = tango1_gradient(varargin{1}, varargin{2}, m);
    for j = 2:n
      map{j} = tango1_gradient(varargin{j}, varargin{j + 1}, m + 1);
      map{j}(1, :) = [];
    end
    map = cat(1, map{:});
  end

function map = tango1_gradient(lo, hi, m)

  lo = tango_color(lo);
  hi = tango_color(hi);

  map = gray(m) * diag(hi - lo) + lo(ones(m, 1), :);

% tango/color -- color values.
%
%      c = tango('color', name)
%
% Map color name to numeric color value.
%
% Argument NAME is a color name (a string).  When looking up
%  colors, only letters and digits in NAME are considered and
%  case is ignored, too.
%
% Return value is the RGB color value.
function c = tango_color(name)

  persistent p_name p_color
  if isempty(p_name)
    dat = {'black',           tango_black; ...
           'white',           tango_white; ...
           'yellow1',         tango_yellow1; ...
           'lightyellow',     tango_yellow1; ...
           'butter1',         tango_yellow1; ...
           'lightbutter',     tango_yellow1; ...
           'yellow2',         tango_yellow2; ...
           'yellow',          tango_yellow2; ...
           'butter2',         tango_yellow2; ...
           'butter',          tango_yellow2; ...
           'yellow3',         tango_yellow3; ...
           'darkyellow',      tango_yellow3; ...
           'butter3',         tango_yellow3; ...
           'darkbutter',      tango_yellow3; ...
           'orange1',         tango_orange1; ...
           'lightorange',     tango_orange1; ...
           'orange2',         tango_orange2; ...
           'orange',          tango_orange2; ...
           'orange3',         tango_orange3; ...
           'darkorange',      tango_orange3; ...
           'red1',            tango_red1; ...
           'lightred',        tango_red1; ...
           'scarletred1',     tango_red1; ...
           'lightscarletred', tango_red1; ...
           'scarlet1',        tango_red1; ...
           'lightscarlet',    tango_red1; ...
           'red2',            tango_red2; ...
           'red',             tango_red2; ...
           'scarletred2',     tango_red2; ...
           'scarletred',      tango_red2; ...
           'scarlet2',        tango_red2; ...
           'scarlet',         tango_red2; ...
           'red3',            tango_red3; ...
           'darkred',         tango_red3; ...
           'scarletred3',     tango_red3; ...
           'darkscarletred',  tango_red3; ...
           'scarlet3',        tango_red3; ...
           'darkscarlet',     tango_red3; ...
           'purple1',         tango_purple1; ...
           'lightpurple',     tango_purple1; ...
           'plum1',           tango_purple1; ...
           'lightplum',       tango_purple1; ...
           'purple2',         tango_purple2; ...
           'purple',          tango_purple2; ...
           'plum2',           tango_purple2; ...
           'plum',            tango_purple2; ...
           'purple3',         tango_purple3; ...
           'darkpurple',      tango_purple3; ...
           'plum3',           tango_purple3; ...
           'darkplum',        tango_purple3; ...
           'blue1',           tango_blue1; ...
           'lightblue',       tango_blue1; ...
           'skyblue1',        tango_blue1; ...
           'lightskyblue',    tango_blue1; ...
           'sky1',            tango_blue1; ...
           'lightsky',        tango_blue1; ...
           'blue2',           tango_blue2; ...
           'blue',            tango_blue2; ...
           'skyblue2',        tango_blue2; ...
           'skyblue',         tango_blue2; ...
           'sky2',            tango_blue2; ...
           'sky',             tango_blue2; ...
           'blue3',           tango_blue3; ...
           'darkblue',        tango_blue3; ...
           'skyblue3',        tango_blue3; ...
           'darkskyblue',     tango_blue3; ...
           'sky3',            tango_blue3; ...
           'darksky',         tango_blue3; ...
           'green1',          tango_green1; ...
           'lightgreen',      tango_green1; ...
           'chameleon1',      tango_green1; ...
           'lightchameleon',  tango_green1; ...
           'green2',          tango_green2; ...
           'green',           tango_green2; ...
           'chameleon2',      tango_green2; ...
           'chameleon',       tango_green2; ...
           'green3',          tango_green3; ...
           'darkgreen',       tango_green3; ...
           'chameleon3',      tango_green3; ...
           'darkchameleon',   tango_green3; ...
           'brown1',          tango_brown1; ...
           'lightbrown',      tango_brown1; ...
           'chocolate1',      tango_brown1; ...
           'lightchocolate',  tango_brown1; ...
           'brown2',          tango_brown2; ...
           'brown',           tango_brown2; ...
           'chocolate2',      tango_brown2; ...
           'chocolate',       tango_brown2; ...
           'brown3',          tango_brown3; ...
           'darkbrown',       tango_brown3; ...
           'chocolate3',      tango_brown3; ...
           'darkchocolate',   tango_brown3; ...
           'gray1',           tango_gray1; ...
           'grey1',           tango_gray1; ...
           'aluminium1',      tango_gray1; ...
           'gray2',           tango_gray2; ...
           'grey2',           tango_gray2; ...
           'aluminium2',      tango_gray2; ...
           'gray3',           tango_gray3; ...
           'grey3',           tango_gray3; ...
           'aluminium3',      tango_gray3; ...
           'gray4',           tango_gray4; ...
           'grey4',           tango_gray4; ...
           'aluminium4',      tango_gray4; ...
           'gray5',           tango_gray5; ...
           'grey5',           tango_gray5; ...
           'aluminium5',      tango_gray5; ...
           'gray6',           tango_gray6; ...
           'grey6',           tango_gray6; ...
           'aluminium6',      tango_gray6};

    p_name = dat(:, 1);
    p_color = dat(:, 2);
  end

  % Check arguments.
  if nargin < 1
    error(nargchk(1, 1, nargin));
  end

  if ~ mtk('stringp', name)
    error('Invalid argument');
  end

  % Lookup color.
  ind = strcmp(lower(name(isletter(name) | (name >= '0' & name <= '9'))), p_name);
  if ~ any(ind)
    error('Unknown color name');
  end

  c = p_color{ind};

% tango/yellow -- shades of yellow.
%
%      c = tango('yellow', brightness)
%
% Return shades of yellow.
%
% Optional argument BRIGHTNESS is either 'light', 'normal',
%  or 'dark'.  Default is to return the normal color value.
%
% Return value is the RGB color value.
function c = tango_yellow(brightness)

  if nargin < 1
    brightness = 'normal';
  end

  switch brightness
   case 'light'
    c = tango_yellow1;
   case 'normal'
    c = tango_yellow2;
   case 'dark'
    c = tango_yellow3;
   otherwise
    error('Invalid argument');
  end

% tango/orange -- shades of orange.
%
%      c = tango('orange', brightness)
%
% Return shades of orange.
%
% Optional argument BRIGHTNESS is either 'light', 'normal',
%  or 'dark'.  Default is to return the normal color value.
%
% Return value is the RGB color value.
function c = tango_orange(brightness)

  if nargin < 1
    brightness = 'normal';
  end

  switch brightness
   case 'light'
    c = tango_orange1;
   case 'normal'
    c = tango_orange2;
   case 'dark'
    c = tango_orange3;
   otherwise
    error('Invalid argument');
  end

% tango/red -- shades of red.
%
%      c = tango('red', brightness)
%
% Return shades of red.
%
% Optional argument BRIGHTNESS is either 'light', 'normal',
%  or 'dark'.  Default is to return the normal color value.
%
% Return value is the RGB color value.
function c = tango_red(brightness)

  if nargin < 1
    brightness = 'normal';
  end

  switch brightness
   case 'light'
    c = tango_red1;
   case 'normal'
    c = tango_red2;
   case 'dark'
    c = tango_red3;
   otherwise
    error('Invalid argument');
  end

% tango/purple -- shades of purple.
%
%      c = tango('purple', brightness)
%
% Return shades of purple.
%
% Optional argument BRIGHTNESS is either 'light', 'normal',
%  or 'dark'.  Default is to return the normal color value.
%
% Return value is the RGB color value.
function c = tango_purple(brightness)

  if nargin < 1
    brightness = 'normal';
  end

  switch brightness
   case 'light'
    c = tango_purple1;
   case 'normal'
    c = tango_purple2;
   case 'dark'
    c = tango_purple3;
   otherwise
    error('Invalid argument');
  end

% tango/blue -- shades of blue.
%
%      c = tango('blue', brightness)
%
% Return shades of blue.
%
% Optional argument BRIGHTNESS is either 'light', 'normal',
%  or 'dark'.  Default is to return the normal color value.
%
% Return value is the RGB color value.
function c = tango_blue(brightness)

  if nargin < 1
    brightness = 'normal';
  end

  switch brightness
   case 'light'
    c = tango_blue1;
   case 'normal'
    c = tango_blue2;
   case 'dark'
    c = tango_blue3;
   otherwise
    error('Invalid argument');
  end

% tango/green -- shades of green.
%
%      c = tango('green', brightness)
%
% Return shades of green.
%
% Optional argument BRIGHTNESS is either 'light', 'normal',
%  or 'dark'.  Default is to return the normal color value.
%
% Return value is the RGB color value.
function c = tango_green(brightness)

  if nargin < 1
    brightness = 'normal';
  end

  switch brightness
   case 'light'
    c = tango_green1;
   case 'normal'
    c = tango_green2;
   case 'dark'
    c = tango_green3;
   otherwise
    error('Invalid argument');
  end

% tango/brown -- shades of brown.
%
%      c = tango('brown', brightness)
%
% Return shades of brown.
%
% Optional argument BRIGHTNESS is either 'light', 'normal',
%  or 'dark'.  Default is to return the normal color value.
%
% Return value is the RGB color value.
function c = tango_brown(brightness)

  if nargin < 1
    brightness = 'normal';
  end

  switch brightness
   case 'light'
    c = tango_brown1;
   case 'normal'
    c = tango_brown2;
   case 'dark'
    c = tango_brown3;
   otherwise
    error('Invalid argument');
  end

% tango/light_gray -- shades of light gray.
%
%      c = tango('light_gray', brightness)
%
% Return shades of light gray.
%
% Optional argument BRIGHTNESS is either 'light', 'normal',
%  or 'dark'.  Default is to return the normal color value.
%
% Return value is the RGB color value.
function c = tango_light_gray(brightness)

  if nargin < 1
    brightness = 'normal';
  end

  switch brightness
   case 'light'
    c = tango_gray1;
   case 'normal'
    c = tango_gray2;
   case 'dark'
    c = tango_gray3;
   otherwise
    error('Invalid argument');
  end

% tango/dark_gray -- shades of dark gray.
%
%      c = tango('dark_gray', brightness)
%
% Return shades of dark gray.
%
% Optional argument BRIGHTNESS is either 'light', 'normal',
%  or 'dark'.  Default is to return the normal color value.
%
% Return value is the RGB color value.
function c = tango_dark_gray(brightness)

  if nargin < 1
    brightness = 'normal';
  end

  switch brightness
   case 'light'
    c = tango_gray4;
   case 'normal'
    c = tango_gray5;
   case 'dark'
    c = tango_gray6;
   otherwise
    error('Invalid argument');
  end

% Tango color palette.
function c = tango_black,   c = [  0,   0,   0];
function c = tango_white,   c = [  1,   1,   1];

function c = tango_yellow1, c = [252, 233,  79] ./ 255;
function c = tango_yellow2, c = [237, 212,   0] ./ 255;
function c = tango_yellow3, c = [196, 160,   0] ./ 255;
function c = tango_orange1, c = [252, 175,  62] ./ 255;
function c = tango_orange2, c = [245, 121,   0] ./ 255;
function c = tango_orange3, c = [206,  92,   0] ./ 255;
function c = tango_red1,    c = [239,  41,  41] ./ 255;
function c = tango_red2,    c = [204,   0,   0] ./ 255;
function c = tango_red3,    c = [164,   0,   0] ./ 255;
function c = tango_purple1, c = [173, 127, 168] ./ 255;
function c = tango_purple2, c = [117,  80, 123] ./ 255;
function c = tango_purple3, c = [ 92,  53, 102] ./ 255;
function c = tango_blue1,   c = [114, 159, 207] ./ 255;
function c = tango_blue2,   c = [ 52, 101, 164] ./ 255;
function c = tango_blue3,   c = [ 32,  74, 135] ./ 255;
function c = tango_green1,  c = [138, 226,  52] ./ 255;
function c = tango_green2,  c = [115, 210,  22] ./ 255;
function c = tango_green3,  c = [ 78, 154,   6] ./ 255;
function c = tango_brown1,  c = [233, 185, 110] ./ 255;
function c = tango_brown2,  c = [193, 125,  17] ./ 255;
function c = tango_brown3,  c = [143,  89,   2] ./ 255;
function c = tango_gray1,   c = [238, 238, 236] ./ 255;
function c = tango_gray2,   c = [211, 215, 207] ./ 255;
function c = tango_gray3,   c = [186, 189, 182] ./ 255;
function c = tango_gray4,   c = [136, 138, 133] ./ 255;
function c = tango_gray5,   c = [ 85,  87,  83] ./ 255;
function c = tango_gray6,   c = [ 46,  52,  54] ./ 255;

%% tango.m ends here
